/**********************************************************************
 * Copyright (c) 2003 Hyades project.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v0.5
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v05.html
 * 
 * Contributors: 
 * Scapa - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.execution.remote;

import org.eclipse.hyades.internal.execution.local.common.CustomCommand;
import org.eclipse.hyades.internal.execution.remote.AgentControllerListener;
import org.eclipse.hyades.internal.execution.remote.AgentControllerUnavailableException;
import org.eclipse.hyades.internal.execution.remote.CustomCommandHandler;
import org.eclipse.hyades.internal.execution.remote.RemoteClientListener;
import org.eclipse.hyades.internal.execution.remote.RemoteComponentSkeleton;

public class RemoteSession  {

	/* Fields for controlling exit */
	private Object completionLock=new Object();
	private boolean completed=false;
	
	/* The agent we use to communicate with the client */
	RemoteComponentSkeleton agent;
	

	public RemoteSession(String name, String type) {
		agent=new RemoteComponentSkeleton(name,type);
	}
	
	/**
	 * 
	 *
	 */
	public void init() {
		
		/* Add a listener for RAC events */
		agent.addAgentControllerListener(new AgentControllerListener() {

			public void agentControllerActive() {
				// TODO Auto-generated method stub
				
			}

			public void agentControllerInactive() {
				// TODO Auto-generated method stub
				
			}
		});
		
		/* Add a listener for events when the client attaches/detaches */
		agent.addRemoteClientListener(new RemoteClientListener() {

			public void clientActive() {
				// TODO Auto-generated method stub
				/* RKD:  This method will never be invoked at this point.  It is
				 * not currently implemented in the RAC but I feel it will be needed.
				 */
				
			}

			public void clientInactive() {
				// TODO Auto-generated method stub
				
			}
		});
		
		/*  Add our command handler */
		agent.addCommandListener(new CustomCommandHandler() {

			public void handleCommand(CustomCommand command) {
				/* This is where our agent is given the commands from
				 * the remote client
				 */
				
			}	
		});
	
		/* try and handshake with the RAC */
		try {
			agent.initialize();
		}
		catch(AgentControllerUnavailableException e) {
			/* The RAC is not running */
		}
			
	}
	
	/**
	 *  This method blocks the calling thread until the exit() method
	 * is invoked on another thread
	 */ 
	public void waitForExit() {
		while(!completed) {
			synchronized(completionLock) {
				try {
					completionLock.wait();
				}
				catch(InterruptedException e) {
					/* We can ignore this */
				}
			}
		}
	}
	
	/**
	 *  This method releases the blocked thread inside of waitForExit() method
	 * 
	 */
	private void exit() {
		/* If we lose our RAC connection we will exit */
		synchronized(completionLock) {
			completed=true;
			completionLock.notify();
		}
	}
	
	public void marshalCommand(CustomCommand command) {
		// TODO place demarshalling code
	}
	
	public void demarshalCommand(CustomCommand command) {
		// TODO place demarshalling code   
		
	}

	/**
	 * 
	 * @param args 
	 */
	public static void main(String[] args) {
		try {
			RemoteSession rsession = new RemoteSession("RemoteSession","HyadesSession");
			rsession.init();
			
			/* The message processing thread is a daemon thread and will not block
			 * termination of the JVM.  We need to block until we are put in a 
			 * signaled state to exit.
			 */
			rsession.waitForExit(); 
			
		}
		catch(Throwable e) {
			
		}
	}

}